@@ -38,7 +38,7 @@ orthog <- function(Y, Q)
 
 }
 
-orthog_tf <- function(Y, X)
+orthog_tf_fun <- function(Y, X)
 {
   
   Q = tf$linalg$qr(X, full_matrices=FALSE, name="QR")$q
@@ -47,6 +47,15 @@ orthog_tf <- function(Y, X)
   
 }
 
+orthog_tf <- function(Y, X, deactivate_at_test = TRUE)
+{
+  
+  python_path <- system.file("python", package = "deepregression")
+  layers <- reticulate::import_from_path("layers", path = python_path)
+  return(layers$Orthogonalization(deactivate_at_test = deactivate_at_test)(Y, X))
+  
+}
+
 split_model <- function(model, where = -1)
 {
 
@@ -162,7 +171,9 @@ separate_define_relation <- function(
       trmstrings[!no_changes] <- mapply(function(x,y) paste0(x, y), 
                                         trmstrings[!no_changes],
                                         oz_to_add[!no_changes])
-      form <- as.formula(paste0("~ ", paste(trmstrings, collapse = " + ")))
+      formf <- paste(trmstrings, collapse = " + ")
+      if(!has_intercept) formf <- paste0("-1 + ", formf)
+      form <- as.formula(paste0("~ ", formf))
       return(separate_define_relation(form, specials = specials, 
                                       specials_to_oz = specials_to_oz,
                                       automatic_oz_check = FALSE))
